package be.ulg.hoessen.martinLeon.crawler;

import be.ulg.hoessen.martinLeon.chargeurs.ChargeurException;
import be.ulg.hoessen.martinLeon.chargeurs.ChargeurTexte;
import be.ulg.hoessen.martinLeon.indexeur.Index;
import be.ulg.hoessen.martinLeon.Util;
import be.ulg.hoessen.martinLeon.traitement.ConstruireIndex;
import java.io.File;
import java.net.URL;
import java.util.ArrayList;

/**
 * Le but de cette classe est de parcourir les différentes pages web. Elle
 * s'exécutera plusieurs fois en parallèle.
 * @author benjamin, benoit
 */
public class Robot implements Runnable {

    private Index index;
    private Thread processus;
    private Crawler crawler;
    private URL url;
    private int distance;
    private boolean fini,  traiterMots;
    private ArrayList<URLEnAttente> listeEnAttente;
    private volatile boolean mort;
    private ChargeurTexte chargeur;
    private ConstruireIndex traiteur;
    private static int ID = 0;
    private File outputFile;
    

    /**
     * Crée une nouvelle instance de robot de parcours
     * @param unCrawler, le crawler qui a créé ce robot
     * @param unIndex, l'index que l'on mettra à jour
     */
    public Robot(Crawler unCrawler, Index unIndex) {
        index = unIndex;
        crawler = unCrawler;
        listeEnAttente = new ArrayList<URLEnAttente>();
        fini = true;
        mort = false;
        processus = new Thread(this);
        processus.setName("Robot d'indexage "+Robot.ID);
        Robot.ID++;
        processus.start();
        traiteur = new ConstruireIndex(this, unIndex, null, false);
        chargeur = new ChargeurTexte();
        
    }

    /**
     * Methode a implementer pour lancer l'execution en parallele.
     */
    public void run() {
        while (!mort) {
            if (!fini) {
                URLEnAttente tmp;
                synchronized (listeEnAttente) {
                    tmp = listeEnAttente.remove(listeEnAttente.size() - 1);
                }
                if (Util.verbose) {
                    if (tmp.traiterMots()) {
                        System.out.println(processus.getName()+": indexe: " + tmp.url().toString());
                    } else {
                        System.out.println(processus.getName()+": recherche les url dans: " +
                                tmp.url().toString());
                    }
                }
                url = tmp.url();
                distance = tmp.distance();
                traiterMots = tmp.traiterMots();
                try {
                    //chargeur = null;
                    //chargeur = new ChargeurTexte(url);
                    chargeur.initialiserFlux(url);
                    traiteur.setUrl(url);
                    traiteur.traiter(traiterMots);
                    chargeur.setTraiteur(traiteur);
                    chargeur.charger();
                } catch (ChargeurException ex) {
                    if (Util.verbose) {
                        ex.printStackTrace();
                    }
                } finally {
                    crawler.pageParcourue();
                    synchronized (listeEnAttente) {
                        fini = listeEnAttente.size() == 0;
                    }
                    if (fini) {
                        crawler.signalFini(this);
                    }
                }
            } else {
                synchronized (listeEnAttente) {
                    fini = listeEnAttente.size() == 0;
                }
                try {
                    Thread.sleep(50);
                } catch (InterruptedException ex) {
                    ex.printStackTrace();
                }
            }
        }
    }

    /**
     * Permet de signaler que le robot n'effectuera plus traitement d'url
     * @throws RobotException
     */
    public void tuer() throws RobotException {
        synchronized (listeEnAttente) {
            fini = listeEnAttente.size() == 0;
        }
        if(!fini) throw new RobotException("Le robot doit encore traiter des" +
                " pages");
        mort = true;
    }

    /**
     * On lance le parcours et indexage d'une page
     * @param unUrl, l'url à parcourir
     * @param uneDistance, la distance entre cette page et la page d'origine
     * @param tratier, true si l'on doit indexer les mots
     * @throws RobotException si on tente de lancer le robot alors qu'il tourne
     * déjà
     */
    public void lancer(URLEnAttente a) throws
            RobotException {
        if (mort) {
            //crawler.signalFini(this);
            throw new RobotException("On tente de lancer le robot alors " +
                    "qu'il est mort.");
        } else {

            synchronized (listeEnAttente) {
                listeEnAttente.add(a);
            }
        //url = a.url();
        //distance = a.distance();
        //System.out.println("Distance passee en parametre: "+distance );
        //traiterMots = a.traiterMots();

        }
    }

    /**
     * Methode permettant de signaler au crawler que l'on doit peut etre
     * parcourir l'url passe en parametre
     * @param unUrl l'url qui doit peut-etre etre parcouru
     */
    public void parcourirUrl(URL unUrl) {
        int tmp = distance;
        tmp++;
//System.out.println("Distance que l'on envoie: "+tmp);
        crawler.parcourirUrl(unUrl, tmp, traiterMots);
    }

    @Override
    public String toString() {
        if (!mort && !fini) {
            return "occup\u00e9 de parcourir l'URL " + url.toString();
        } else if (!mort && fini) {
            return "en attente d'une nouvel URL";
        } else {
            return "a fini de travailler";
        }
    }
}
